
/*
 * $Id: StatPing.cc,v 1.1.1.2 1999/02/05 16:51:40 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iomanip.h>

#include "Clock.h"
#include "StatPing.h"
#include "StatPhaseMgr.h"
#include "Progress.h"
#include "ILog.h"
#include "LogComment.h"
#include "logTags.h"
#include "gadgets.h"
#include "globals.h"

/* StatPingRec */

StatPingRec::StatPingRec() {
	theSelCnt = theRdCnt = theWrCnt = 0;
	theRdSz = theWrSz = 0;
	theHitCnt = theMissCnt = 0;
	theHitSz = theMissSz = 0;
	theErrCnt = 0;
}

void StatPingRec::store(OLog &log) const {
	Assert(theDuration >= 0);
	log
		<< theXactLvl << theOpenLvl << theEstbLvl
		<< theSelCnt << theRdCnt << theWrCnt
		<< theRdSz << theWrSz
		<< theHitCnt << theMissCnt 
		<< theHitSz << theMissSz 
		<< theErrCnt
		<< theDuration
		;
}

void StatPingRec::load(ILog &log) {
	log
		>> theXactLvl >> theOpenLvl >> theEstbLvl
		>> theSelCnt >> theRdCnt >> theWrCnt
		>> theRdSz >> theWrSz
		>> theHitCnt >> theMissCnt 
		>> theHitSz >> theMissSz 
		>> theErrCnt
		>> theDuration
		;
}

void StatPingRec::restart() {
	theXactLvl.restart();
	theOpenLvl.restart();
	theEstbLvl.restart();
	theSelCnt = theRdCnt = theWrCnt = 0;
	theRdSz = theWrSz = 0;
	theHitCnt = theMissCnt = 0;
	theHitSz = theMissSz = 0;
	theErrCnt = 0;
}


/* StatPing */

StatPing::StatPing(Time aPeriod): StatClockInt(aPeriod) {
}

void StatPing::flush() {
	theDuration = TheClock - theIntStart;

	OLog &log = TheLog();
	log << bege(lgStatPing);
	StatPingRec::store(log);
	log << ende;

	if (theDuration > 0)
		report();
}

void StatPing::restart() {
	StatClockInt::restart();
	StatPingRec::restart();
}

void StatPing::report() const {
	Comment(1)
		<< "i-" << TheStatPhaseMgr->name()
		<< ' ' << setw(6) << TheProgress.theXactComplCnt
		<< ' ' << setw(6) << Ratio(theXactLvl.decCnt(), theDuration.secd())
		<< ' ' << setw(6) << Percent(theHitCnt, theHitCnt + theMissCnt)
		<< ' ' << setw(3) << theErrCnt
		<< endc;
}

