
/*
 * $Id: World.cc,v 1.1.1.2 1999/02/05 16:51:41 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <math.h>
//#include <ctype.h>
#include <fstream.h>

#include "Rnd.h"
#include "World.h"
#include "LogComment.h"
#include "SharedOpts.h"
#include "gadgets.h"

World TheWorld;

// Euler-Mascheroni constant
#define TheEMConst 0.57721566490153286060651209


World::World(): theCapacity(0), theZipfOmega(-1) {
}

World::~World() {
	while (theUrls.count())
		delete[] theUrls.pop();
}

void World::syncZipfOmega() {
	// harmonic sum approximation: log(i) + gamma + 1/(2i)
	theZipfOmega = log(theCapacity) + TheEMConst + 1./(2*theCapacity);
}

int World::genOId() const {
	// XXX: hard coded zipf(1) distribution
	static RndGen theRnd(TheOpts.theRngSeed);
	return (int) (0.5 + exp(theZipfOmega*theRnd() - TheEMConst));
}

bool World::loadUrls(const char *fname) {
	char url[8*1024];

	Assert(fname);

	ifstream is(fname);
	Size sz = 0;
	int lineCount = 0;
	int urlCount = 0;

	Comment(1) << "loading urls from '" << fname << "'" << endc;

	while (is && is.getline(url, sizeof(url))) {
		lineCount++;
		const int len = strlen(url);
		if (len && *url == '/') {
			theUrls.append(StrDup(url));
			sz += sizeof(char*) + len + 1;
			urlCount++;
		} else
		if (len && *url != '#') {
			Comment << fname << ':' << lineCount << " unknown url format near " << url << endc;
			return false;
		}
	}

	is.close();

	Comment(1)
		<< "loaded " << urlCount 
		<< " urls from '" << fname << "'"
		<< " (" << lineCount << " lines)"
		<< " consumed ~ " << sz/(1024*1024.) << " MB" << endc;

	return urlCount > 0;
}

bool World::genObjIds(int count, int order, bool unique) {
	// pump up so we do not get wrapped at the end of the experiment
	// the latter may cause hits with miss-only workload
	count += count/20; // 5%

	theObjIds.stretch(count);
	for (int i = 0; i < count; ++i)
		theObjIds.append(genOId());

	// remember positions if needed
	if (unique) {
		theUnqIds.stretch(count);
		for (int i = 1; i <= count; ++i)
			theUnqIds.append(i);
	}

	// perform permutation if needed
	if (order != 1) { // note: order == 1 -- initial order
		RndGen rng(order);
		for (int i = 0; i < count; ++i) {
			const int newOff = rng(0, count);
			theObjIds.swap(i, newOff);
			if (unique)
				theUnqIds.swap(i, newOff);
		}
	}

	return theObjIds.count() >= count;
}
