
/*
 * $Id: Farm.h,v 1.1.1.3 1999/02/07 05:08:47 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef FARM_H
#define FARM_H

#include "Array.h"

// a collection of Items that configures and recycles its members
// current recycling order is LIFO
// note <Item> -> <Item*> migration
template <class Item>
class Farm: public Array<Item*> {
	public:
		Farm(): theOutLevel(0) {}
		virtual ~Farm() {}// g++ bug: { while (!empty()) destroy(get()); }

		void limit(int limit) { stretch(limit); }

		int capacity() const { return Array<Item*>::capacity(); }
		int level() const { return count(); }

		Item *get() { theOutLevel++; return conf(empty() ? gen() : pop()); }
		void put(Item *i) { theOutLevel--; full() ? overflow(i) : push(i); }

	protected:
		virtual Item *gen() = 0;
		virtual Item *conf(Item *i) const { return i; }
		virtual void destroy(Item *i) = 0;

		virtual void overflow(Item *i) { destroy(i); }

	protected:
		int theOutLevel;    // items out there
};

// Fixed capacity Farm for "objects" (new/reset/delete)
template <class Item>
class ObjFarm: public Farm<Item> {
	public:
		ObjFarm() {}
		virtual ~ObjFarm() { while (!empty()) destroy(get()); }// g++ bug: {}

	protected:
		virtual Item *gen() { return new Item(); }
		virtual Item *conf(Item *i) const { i->reset(); return i; }
		virtual void destroy(Item *i) { delete i; }
};

#endif
