
/*
 * $Id: Time.cc,v 1.1.1.3 1999/02/05 16:51:56 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <stdlib.h>
#include <iostream.h>
#include <iomanip.h>

#include "Time.h"


Time Time::Now() {
	Time tm;
	gettimeofday(&tm, 0);
	return tm;
}

Time Time::Msec(long msec) {
	return Time(msec / 1000, 1000*(msec % 1000));
}

Time Time::Secd(double dsec) {
	const long sec = (long)dsec;
	return Time(sec, (long)((dsec-sec)*1e6));
}

Time &Time::operator +=(const Time &tm) {

	tv_sec += tm.tv_sec;
	tv_usec += tm.tv_usec;

	if (tv_usec >= 1000000L) {
		tv_usec -= 1000000L;
		tv_sec++;
	}

	return *this;
}

// note: negative times are confusing for humans; e.g., "-1:1" means "-0.1"
Time &Time::operator -=(const Time &tm) {

	tv_sec -= tm.tv_sec;
	tv_usec -= tm.tv_usec;

	if (tv_usec < 0) {
		tv_usec += 1000000L;
		tv_sec--;
	}

	return *this;
}

Time &Time::operator *=(int factor) {
	tv_sec *= factor;
	tv_usec *= factor;

	if (tv_usec >= 1000000L) {
		tv_sec += tv_usec/1000000L;
		tv_usec %= 1000000L;
	} else
	if (tv_usec < 0) {
		tv_usec = -tv_usec;
		tv_sec -= tv_usec/1000000L;
		tv_sec--;
		tv_usec = 1000000L - (tv_usec % 1000000L);
	}

	return *this;
}

Time &Time::operator /=(double factor) {
	if (factor) {
		const double dsec = secd()/factor;
		tv_sec = (long) dsec;
		tv_usec = (long)((dsec-tv_sec)*1e6);
	} else {
		tv_sec = tv_usec = -1;
	}

	return *this;
}

istream &Time::read(istream &is) {
	char c;
	return is >> tv_sec >> c >> tv_usec;
}

ostream &Time::print(ostream &os) const {
	if (tv_sec < 60*60*24*365 && tv_usec >= 0)
		return printInterval(os);
	if (tv_sec == -1 && tv_usec == -1)
		return os << "<none>"; // works for everybody?

	const char osfill = os.fill();
	return os << tv_sec << '.' << setw(6) << setfill('0') << tv_usec << setfill(osfill);
}

ostream &Time::printInterval(ostream &os) const {

	const Time &t = *this; // historical artifact

	const int osprec = os.precision(2);
	const char osfill = os.fill();

	if (t.tv_sec < 0)
		os << t.tv_sec << '.' << setfill('0') << setw(6) << t.tv_usec << " sec";
	else
	if (t.tv_sec == 0)
		if (t.tv_usec < 1000)
			os << t.tv_usec << " usec";
		else
			os << (t.tv_usec/1000) << " msec";
	else
	if (t.tv_sec < 60)
		os << t.tv_sec << '.' << setfill('0') << setw(3) << (t.tv_usec/1000) << " sec";
	else
	if (t.tv_sec < 60*60)
		os << (t.tv_sec/60) << '.' << (10*(t.tv_sec%60)/60) << " min";
	else
		os << (t.tv_sec/(60*60)) << '.' << (10*(t.tv_sec%(60*60))/(60*60)) << " hrs";

	os.fill(osfill);
	os.precision(osprec);

	return os;
}
