#ifndef X_GADGETS_H
#define X_GADGETS_H

/*
 * $Id: gadgets.h,v 1.1.1.2 1999/01/30 17:39:38 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <stdlib.h>
#include <string.h>

class ostream;

// abs, min, and max
template <class T> inline T Abs(T a) { return a > 0 ? a : -a; }
template <class T> inline T Min(T a, T b) { return (a <= b) ? a : b; }
template <class T> inline T Max(T a, T b) { return (a >= b) ? a : b; }
template <class T> inline T Min(T a, T b, T c) { return Min(Min(a,b), c); }
template <class T> inline T Max(T a, T b, T c) { return Max(Max(a,b), c); }

// safe division
inline double Ratio(int n, double d) { return d ? (n ? n/d : 0) : -1; }
inline double Ratio(double n, double d) { return d ? n/d : -1; }
inline double Percent(int n, double d) { return Ratio(n, d/100); }

// new/delete[] style string duplication
extern char *StrDup(const char *str, int len); // dups exactly len chars and appends '\0'
inline char *StrDup(const char *str) { return str ? StrDup(str, strlen(str)) : 0; }

// a better atoi, returns def on error
extern int xatoi(const char *s, int def = 0);

// similar to strto*-like functions
// note: isDbl fails on integers, isNum does not
extern bool isInt(const char *s, int &i, const char **p = 0);
extern bool isDbl(const char *s, double &d, const char **p = 0);
extern bool isNum(const char *s, double &d, const char **p = 0);

// set ``default'' format for a stream and custom precision
void configureStream(ostream &os, int prec);

// returns maximum number of FDs allowed
extern int GetMaxFD();
// attempts to set max number of FDs; returns actual maxFD set or -1 on failure
extern int SetMaxFD(int maxFD);

#endif
